# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import unittest

from typing import (
    Any,
    Callable,
    ClassVar,
    Dict,
    Generic,
    Iterable,
    List,
    Mapping,
    MutableMapping,
    NamedTuple,
    Optional,
    Sequence,
    Tuple,
    TypeVar,
    Union,
)
from azure_functions_runtime_v1.utils.typing_inspect import (
    get_args,
    get_generic_bases,
    get_generic_type,
    get_origin,
    get_parameters,
    is_callable_type,
    is_classvar,
    is_generic_type,
    is_tuple_type,
    is_typevar,
    is_union_type,
)


class IsUtilityTestCase(unittest.TestCase):
    def sample_test(self, fun, samples, nonsamples):
        for s in samples:
            self.assertTrue(fun(s), f"{s} type expected in {samples}")
        for s in nonsamples:
            self.assertFalse(fun(s), f"{s} type expected in {nonsamples}")

    def test_generic(self):
        T = TypeVar('T')
        samples = [Generic, Generic[T], Iterable[int], Mapping,
                   MutableMapping[T, List[int]], Sequence[Union[str, bytes]]]
        nonsamples = [int, Union[int, str], Union[int, T], ClassVar[List[int]],
                      Callable[..., T], ClassVar, Optional, bytes, list]
        self.sample_test(is_generic_type, samples, nonsamples)

    def test_callable(self):
        class MyClass(Callable[[int], int]):
            pass
        samples = [Callable, Callable[..., int],
                   Callable[[int, int], Iterable[str]]]
        nonsamples = [int, type, 42, [], List[int],
                      Union[callable, Callable[..., int]]]
        self.sample_test(is_callable_type, samples, nonsamples)
        self.assertTrue(is_callable_type(MyClass))

    def test_tuple(self):
        class MyClass(Tuple[str, int]):
            pass
        samples = [Tuple, Tuple[str, int], Tuple[Iterable, ...]]
        nonsamples = [int, tuple, 42, List[int], NamedTuple('N', [('x', int)])]
        self.sample_test(is_tuple_type, samples, nonsamples)
        self.assertTrue(is_tuple_type(MyClass))

    def test_union(self):
        T = TypeVar('T')
        S = TypeVar('S')
        samples = [Union, Union[T, int], Union[int, Union[T, S]]]
        nonsamples = [int, Union[int, int], [], Iterable[Any]]
        self.sample_test(is_union_type, samples, nonsamples)

    def test_typevar(self):
        T = TypeVar('T')
        S_co = TypeVar('S_co', covariant=True)
        samples = [T, S_co]
        nonsamples = [int, Union[T, int], Union[T, S_co], type, ClassVar[int]]
        self.sample_test(is_typevar, samples, nonsamples)

    def test_classvar(self):
        T = TypeVar('T')
        samples = [ClassVar, ClassVar[int], ClassVar[List[T]]]
        nonsamples = [int, 42, Iterable, List[int], type, T]
        self.sample_test(is_classvar, samples, nonsamples)


class GetUtilityTestCase(unittest.TestCase):

    def test_origin(self):
        T = TypeVar('T')

        class MyClass(Generic[T]):
            pass

        self.assertEqual(get_origin(int), None)
        self.assertEqual(get_origin(ClassVar[int]), None)
        self.assertEqual(get_origin(Generic), Generic)
        self.assertEqual(get_origin(Generic[T]), Generic)
        self.assertEqual(get_origin(List[Tuple[T, T]][int]), list)
        self.assertEqual(get_origin(MyClass), None)

    def test_parameters(self):
        T = TypeVar('T')
        S_co = TypeVar('S_co', covariant=True)
        U = TypeVar('U')
        self.assertEqual(get_parameters(int), ())
        self.assertEqual(get_parameters(Generic), ())
        self.assertEqual(get_parameters(Union), ())
        self.assertEqual(get_parameters(List[int]), ())
        self.assertEqual(get_parameters(Generic[T]), (T,))
        self.assertEqual(get_parameters(Tuple[List[T], List[S_co]]), (T, S_co))
        self.assertEqual(get_parameters(Union[S_co, Tuple[T, T]][int, U]), (U,))
        self.assertEqual(get_parameters(Mapping[T, Tuple[S_co, T]]), (T, S_co))

    def test_args_evaluated(self):
        T = TypeVar('T')
        self.assertEqual(get_args(Union[int, Tuple[T, int]][str], evaluate=True),
                         (int, Tuple[str, int]))
        self.assertEqual(get_args(Dict[int, Tuple[T, T]][Optional[int]], evaluate=True),
                         (int, Tuple[Optional[int], Optional[int]]))
        self.assertEqual(get_args(Callable[[], T][int], evaluate=True), ([], int,))

    def test_generic_type(self):
        T = TypeVar('T')

        class Node(Generic[T]):
            pass
        self.assertIs(get_generic_type(Node()), Node)
        self.assertIs(get_generic_type(Node[int]()), Node[int])
        self.assertIs(get_generic_type(Node[T]()), Node[T],)
        self.assertIs(get_generic_type(1), int)

    def test_generic_bases(self):
        class MyClass(List[int], Mapping[str, List[int]]):
            pass
        self.assertEqual(get_generic_bases(MyClass),
                         (List[int], Mapping[str, List[int]]))
        self.assertEqual(get_generic_bases(int), ())



from __future__ import annotations

from concurrent.futures import ThreadPoolExecutor
from typing import Optional

from .app_setting_manager import get_app_setting
from .constants import (
    PYTHON_THREADPOOL_THREAD_COUNT,
    PYTHON_THREADPOOL_THREAD_COUNT_DEFAULT,
    PYTHON_THREADPOOL_THREAD_COUNT_MIN,
    PYTHON_THREADPOOL_THREAD_COUNT_MAX,
)
from ..logging import logger

_threadpool_executor: Optional[ThreadPoolExecutor] = None


def _validate_thread_count(value: str) -> bool:
    try:
        int_value = int(value)
    except ValueError:
        logger.warning('%s must be an integer', PYTHON_THREADPOOL_THREAD_COUNT)
        return False

    if (int_value < PYTHON_THREADPOOL_THREAD_COUNT_MIN
            or int_value > PYTHON_THREADPOOL_THREAD_COUNT_MAX):
        logger.warning(
            '%s must be set to a value between %s and %s. Reverting to '
            'default value (%s).',
            PYTHON_THREADPOOL_THREAD_COUNT,
            PYTHON_THREADPOOL_THREAD_COUNT_MIN,
            PYTHON_THREADPOOL_THREAD_COUNT_MAX,
            PYTHON_THREADPOOL_THREAD_COUNT_DEFAULT,
        )
        return False
    return True


def _get_max_workers() -> Optional[int]:
    threadpool_count = get_app_setting(
        setting=PYTHON_THREADPOOL_THREAD_COUNT,
        validator=_validate_thread_count,
    )
    if threadpool_count is None:
        return None
    try:
        return int(threadpool_count)
    except (TypeError, ValueError) as e:
        logger.warning(
            'Failed to convert %s value "%s" to integer: %s',
            PYTHON_THREADPOOL_THREAD_COUNT, threadpool_count, e
        )
        return None


def start_threadpool_executor() -> None:
    global _threadpool_executor
    max_workers = _get_max_workers()

    if _threadpool_executor is not None:
        try:
            _threadpool_executor.shutdown(wait=False)
        except Exception:
            pass

    _threadpool_executor = ThreadPoolExecutor(max_workers=max_workers)
    logger.debug(
        'Started threadpool executor (id=%s) with max_workers=%s',
        id(_threadpool_executor),
        max_workers,
    )


def stop_threadpool_executor() -> None:
    global _threadpool_executor
    if _threadpool_executor is not None:
        try:
            _threadpool_executor.shutdown(wait=True)
            logger.debug('Stopped threadpool executor (id=%s)',
                         id(_threadpool_executor))
        finally:
            _threadpool_executor = None


def get_threadpool_executor() -> Optional[ThreadPoolExecutor]:
    return _threadpool_executor



from azure_functions_runtime_v1.utils import threadpool as tp


def _reset():  # helper for clean state
    if tp.get_threadpool_executor() is not None:  # pragma: no cover - cleanup
        tp.stop_threadpool_executor()


def test_start_and_get_threadpool():
    _reset()
    tp.start_threadpool_executor()
    ex = tp.get_threadpool_executor()
    assert ex is not None
    first_id = id(ex)
    tp.start_threadpool_executor()  # restart replaces
    ex2 = tp.get_threadpool_executor()
    assert ex2 is not None and id(ex2) != first_id
    _reset()


def test_stop_threadpool():
    _reset()
    tp.start_threadpool_executor()
    assert tp.get_threadpool_executor() is not None
    tp.stop_threadpool_executor()
    assert tp.get_threadpool_executor() is None


def test_validate_thread_count_invalid(monkeypatch):
    def fake_get_app_setting(setting, validator):
        assert validator("not-int") is False
        return "not-int"
    monkeypatch.setattr(tp, 'get_app_setting', fake_get_app_setting)
    assert tp._get_max_workers() is None


def test_validate_thread_count_range(monkeypatch):
    def fake_get_app_setting(setting, validator):
        assert validator("0") is False
        return "0"
    monkeypatch.setattr(tp, 'get_app_setting', fake_get_app_setting)
    assert tp._get_max_workers() == 0


def test_max_workers_valid(monkeypatch):
    def fake_get_app_setting(setting, validator):
        assert validator("10") is True
        return "10"
    monkeypatch.setattr(tp, 'get_app_setting', fake_get_app_setting)
    assert tp._get_max_workers() == 10



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import unittest

from azure_functions_runtime_v1.logging import format_exception


class TestLogging(unittest.TestCase):

    def test_format_exception(self):
        def call0(fn):
            call1(fn)

        def call1(fn):
            call2(fn)

        def call2(fn):
            fn()

        def raising_function():
            raise ValueError("Value error being raised.", )

        try:
            call0(raising_function)
        except ValueError as e:
            processed_exception = format_exception(e)
            self.assertIn("call0", processed_exception)
            self.assertIn("call1", processed_exception)
            self.assertIn("call2", processed_exception)
            self.assertIn("f", processed_exception)
            self.assertRegex(processed_exception,
                             r".*tests/unittests/test_logging.py.*")



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import unittest

import tests.protos as protos

from datetime import datetime
from http.cookies import SimpleCookie

from azure_functions_runtime_v1.bindings.datumdef import (
    Datum,
    parse_cookie_attr_expires,
    parse_cookie_attr_same_site,
    parse_to_rpc_http_cookie_list,
)
from azure_functions_runtime_v1.bindings.nullable_converters import (
    to_nullable_bool,
    to_nullable_double,
    to_nullable_string,
    to_nullable_timestamp,
)


class TestDatumDef(unittest.TestCase):
    def test_parse_cookie_attr_expires_none(self):
        self.assertEqual(parse_cookie_attr_expires({"expires": None}), None)

    def test_parse_cookie_attr_expires_zero_length(self):
        self.assertEqual(parse_cookie_attr_expires({"expires": ""}), None)

    def test_parse_cookie_attr_expires_valid(self):
        self.assertEqual(parse_cookie_attr_expires(
            {"expires": "Thu, 12 Jan 2017 13:55:08 GMT"}),
            datetime.strptime("Thu, 12 Jan 2017 13:55:08 GMT",
                              "%a, %d %b %Y %H:%M:%S GMT"))

    def test_parse_cookie_attr_expires_value_error(self):
        with self.assertRaises(ValueError):
            parse_cookie_attr_expires(
                {"expires": "Thu, 12 Jan 2017 13:550:08 GMT"})

    def test_parse_cookie_attr_expires_overflow_error(self):
        with self.assertRaises(ValueError):
            parse_cookie_attr_expires(
                {"expires": "Thu, 12 Jan 9999999999999999 13:55:08 GMT"})

    def test_parse_cookie_attr_same_site_default(self):
        self.assertEqual(parse_cookie_attr_same_site(
            {}, protos),
            getattr(protos.RpcHttpCookie.SameSite, "None"))

    def test_parse_cookie_attr_same_site_lax(self):
        self.assertEqual(parse_cookie_attr_same_site(
            {'samesite': 'lax'}, protos),
            getattr(protos.RpcHttpCookie.SameSite, "Lax"))

    def test_parse_cookie_attr_same_site_strict(self):
        self.assertEqual(parse_cookie_attr_same_site(
            {'samesite': 'strict'}, protos),
            getattr(protos.RpcHttpCookie.SameSite, "Strict"))

    def test_parse_cookie_attr_same_site_explicit_none(self):
        self.assertEqual(parse_cookie_attr_same_site(
            {'samesite': 'none'}, protos),
            getattr(protos.RpcHttpCookie.SameSite, "ExplicitNone"))

    def test_parse_to_rpc_http_cookie_list_none(self):
        self.assertEqual(parse_to_rpc_http_cookie_list(None, protos), None)

    @unittest.skip("TODO: fix this test. Figure out what to do with Timestamp")
    def test_parse_to_rpc_http_cookie_list_valid(self):
        headers = [
            'foo=bar; Path=/some/path; Secure; HttpOnly; Domain=123; '
            'SameSite=Lax; Max-Age=12345; Expires=Thu, 12 Jan 2017 13:55:08 '
            'GMT;',
            'foo2=bar; Path=/some/path2; Secure; HttpOnly; Domain=123; '
            'SameSite=Lax; Max-Age=12345; Expires=Thu, 12 Jan 2017 13:55:08 '
            'GMT;']

        cookies = SimpleCookie('\r\n'.join(headers))

        cookie1 = protos.RpcHttpCookie(name="foo",
                                       value="bar",
                                       domain=to_nullable_string("123",
                                                                 "cookie.domain",
                                                                 protos),
                                       path=to_nullable_string("/some/path",
                                                               "cookie.path",
                                                               protos),
                                       expires=to_nullable_timestamp(
                                           parse_cookie_attr_expires(
                                               {
                                                   "expires": "Thu, "
                                                              "12 Jan 2017 13:55:08"
                                                              " GMT"}),
                                           'cookie.expires',
                                           protos),
                                       secure=to_nullable_bool(
                                           bool("True"),
                                           'cookie.secure',
                                           protos),
                                       http_only=to_nullable_bool(
                                           bool("True"),
                                           'cookie.httpOnly',
                                           protos),
                                       same_site=parse_cookie_attr_same_site(
                                           {"samesite": "Lax"},
                                           protos),
                                       max_age=to_nullable_double(
                                           12345,
                                           'cookie.maxAge',
                                           protos))

        cookie2 = protos.RpcHttpCookie(name="foo2",
                                       value="bar",
                                       domain=to_nullable_string("123",
                                                                 "cookie.domain",
                                                                 protos),
                                       path=to_nullable_string("/some/path2",
                                                               "cookie.path",
                                                               protos),
                                       expires=to_nullable_timestamp(
                                           parse_cookie_attr_expires(
                                               {
                                                   "expires": "Thu, "
                                                              "12 Jan 2017 13:55:08"
                                                              " GMT"}),
                                           'cookie.expires',
                                           protos),
                                       secure=to_nullable_bool(
                                           bool("True"),
                                           'cookie.secure',
                                           protos),
                                       http_only=to_nullable_bool(
                                           bool("True"),
                                           'cookie.httpOnly',
                                           protos),
                                       same_site=parse_cookie_attr_same_site(
                                           {"samesite": "Lax"},
                                           protos),
                                       max_age=to_nullable_double(
                                           12345,
                                           'cookie.maxAge',
                                           protos))

        rpc_cookies = parse_to_rpc_http_cookie_list([cookies], protos)
        self.assertEqual(cookie1, rpc_cookies[0])
        self.assertEqual(cookie2, rpc_cookies[1])

    def test_parse_to_rpc_http_cookie_list_no_cookie(self):
        datum = Datum(
            type='http',
            value=dict(
                status_code=None,
                headers=None,
                body=None,
            )
        )

        self.assertIsNone(
            parse_to_rpc_http_cookie_list(datum.value.get('cookies'), protos))



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
from typing import Any, Optional


# This represents the top level protos request sent from the host
class WorkerRequest:
    def __init__(self, name: str, request: Any, properties: dict):
        self.name = name
        self.request = request
        self.properties = properties


# This represents the inner request
class Request:
    def __init__(self, name: Any):
        self.worker_init_request = name
        self.function_load_request = name
        self.function_environment_reload_request = name


# This represents the Function Init/Metadata/Load/Invocation request
class FunctionRequest:
    def __init__(self, capabilities: Optional[Any] = {},
                 function_app_directory: Optional[Any] = "",
                 environment_variables: Optional[Any] = {},
                 function_id: Optional[Any] = "123",
                 metadata: Optional[Any] = {}):
        self.capabilities = capabilities
        self.function_app_directory = function_app_directory
        self.environment_variables = environment_variables
        self.function_id = function_id
        self.metadata = metadata


class Metadata:
    def __init__(self, name: str, directory: str, script_file: str, entry_point: str):
        self.name = name
        self.directory = directory
        self.script_file = script_file
        self.entry_point = entry_point


class MockHttpRequest:
    pass


class MockHttpResponse:
    pass



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import pathlib
import subprocess
import sys
import unittest

ROOT_PATH = pathlib.Path(__file__).parent.parent.parent.parent.parent


class TestCodeQuality(unittest.TestCase):

    def test_flake8(self):
        try:
            import flake8  # NoQA
        except ImportError as e:
            raise unittest.SkipTest('flake8 module is missing') from e

        config_path = ROOT_PATH / '.flake8'
        if not config_path.exists():
            raise unittest.SkipTest('could not locate the .flake8 file')

        project_path = pathlib.Path(ROOT_PATH,'runtimes','v1' ,'azure_functions_runtime_v1')

        try:
            subprocess.run(
                [sys.executable, '-m', 'flake8', '--config', str(config_path),
                 project_path],
                check=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                cwd=str(ROOT_PATH))
        except subprocess.CalledProcessError as ex:
            output = ex.output.decode()
            raise AssertionError(
                'flake8 validation failed:\n%s', output) from None



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import os
import sys
import typing
import unittest
from unittest.mock import patch

from azure_functions_runtime_v1.utils import (app_setting_manager,
                                              helpers,
                                              wrappers)

TEST_APP_SETTING_NAME = "TEST_APP_SETTING_NAME"
TEST_FEATURE_FLAG = "APP_SETTING_FEATURE_FLAG"
FEATURE_DEFAULT = 42


class MockFeature:
    @wrappers.enable_feature_by(TEST_FEATURE_FLAG)
    def mock_feature_enabled(self, output: typing.List[str]) -> str:
        result = 'mock_feature_enabled'
        output.append(result)
        return result

    @wrappers.enable_feature_by(TEST_FEATURE_FLAG, flag_default=True)
    def mock_enabled_default_true(self, output: typing.List[str]) -> str:
        result = 'mock_enabled_default_true'
        output.append(result)
        return result

    @wrappers.disable_feature_by(TEST_FEATURE_FLAG)
    def mock_feature_disabled(self, output: typing.List[str]) -> str:
        result = 'mock_feature_disabled'
        output.append(result)
        return result

    @wrappers.disable_feature_by(TEST_FEATURE_FLAG, flag_default=True)
    def mock_disabled_default_true(self, output: typing.List[str]) -> str:
        result = 'mock_disabled_default_true'
        output.append(result)
        return result

    @wrappers.enable_feature_by(TEST_FEATURE_FLAG, FEATURE_DEFAULT)
    def mock_feature_default(self, output: typing.List[str]) -> str:
        result = 'mock_feature_default'
        output.append(result)
        return result


class MockMethod:
    @wrappers.attach_message_to_exception(ImportError, 'success')
    def mock_load_function_success(self):
        return True

    @wrappers.attach_message_to_exception(ImportError, 'module_not_found')
    def mock_load_function_module_not_found(self):
        raise ModuleNotFoundError('MODULE_NOT_FOUND')

    @wrappers.attach_message_to_exception(ImportError, 'import_error')
    def mock_load_function_import_error(self):
        # ImportError is a subclass of ModuleNotFoundError
        raise ImportError('IMPORT_ERROR')

    @wrappers.attach_message_to_exception(ImportError, 'value_error')
    def mock_load_function_value_error(self):
        # ValueError is not a subclass of ImportError
        raise ValueError('VALUE_ERROR')


class TestUtilities(unittest.TestCase):

    def setUp(self):
        self._dummy_sdk_sys_path = os.path.join(
            os.path.dirname(__file__),
            'resources',
            'mock_azure_functions'
        )

        self.mock_environ = patch.dict('os.environ', os.environ.copy())
        self.mock_sys_module = patch.dict('sys.modules', sys.modules.copy())
        self.mock_sys_path = patch('sys.path', sys.path.copy())
        self.mock_environ.start()
        self.mock_sys_module.start()
        self.mock_sys_path.start()

    def tearDown(self):
        self.mock_sys_path.stop()
        self.mock_sys_module.stop()
        self.mock_environ.stop()

    def test_is_true_like_accepted(self):
        self.assertTrue(app_setting_manager.is_true_like('1'))
        self.assertTrue(app_setting_manager.is_true_like('true'))
        self.assertTrue(app_setting_manager.is_true_like('T'))
        self.assertTrue(app_setting_manager.is_true_like('YES'))
        self.assertTrue(app_setting_manager.is_true_like('y'))

    def test_is_true_like_rejected(self):
        self.assertFalse(app_setting_manager.is_true_like(None))
        self.assertFalse(app_setting_manager.is_true_like(''))
        self.assertFalse(app_setting_manager.is_true_like('secret'))

    def test_is_false_like_accepted(self):
        self.assertTrue(app_setting_manager.is_false_like('0'))
        self.assertTrue(app_setting_manager.is_false_like('false'))
        self.assertTrue(app_setting_manager.is_false_like('F'))
        self.assertTrue(app_setting_manager.is_false_like('NO'))
        self.assertTrue(app_setting_manager.is_false_like('n'))

    def test_is_false_like_rejected(self):
        self.assertFalse(app_setting_manager.is_false_like(None))
        self.assertFalse(app_setting_manager.is_false_like(''))
        self.assertFalse(app_setting_manager.is_false_like('secret'))

    def test_is_envvar_true(self):
        os.environ[TEST_FEATURE_FLAG] = 'true'
        self.assertTrue(app_setting_manager.is_envvar_true(TEST_FEATURE_FLAG))

    def test_is_envvar_not_true_on_unset(self):
        self._unset_feature_flag()
        self.assertFalse(app_setting_manager.is_envvar_true(TEST_FEATURE_FLAG))

    def test_is_envvar_false(self):
        os.environ[TEST_FEATURE_FLAG] = 'false'
        self.assertTrue(app_setting_manager.is_envvar_false(TEST_FEATURE_FLAG))

    def test_is_envvar_not_false_on_unset(self):
        self._unset_feature_flag()
        self.assertFalse(app_setting_manager.is_envvar_true(TEST_FEATURE_FLAG))

    def test_disable_feature_with_no_feature_flag(self):
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_feature_enabled(output)
        self.assertIsNone(result)
        self.assertListEqual(output, [])

    def test_disable_feature_with_default_value(self):
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_disabled_default_true(output)
        self.assertIsNone(result)
        self.assertListEqual(output, [])

    def test_enable_feature_with_feature_flag(self):
        feature_flag = TEST_FEATURE_FLAG
        os.environ[feature_flag] = '1'
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_feature_enabled(output)
        self.assertEqual(result, 'mock_feature_enabled')
        self.assertListEqual(output, ['mock_feature_enabled'])

    def test_enable_feature_with_default_value(self):
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_enabled_default_true(output)
        self.assertEqual(result, 'mock_enabled_default_true')
        self.assertListEqual(output, ['mock_enabled_default_true'])

    def test_enable_feature_with_no_rollback_flag(self):
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_feature_disabled(output)
        self.assertEqual(result, 'mock_feature_disabled')
        self.assertListEqual(output, ['mock_feature_disabled'])

    def test_ignore_disable_default_value_when_set_explicitly(self):
        feature_flag = TEST_FEATURE_FLAG
        os.environ[feature_flag] = '0'
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_disabled_default_true(output)
        self.assertEqual(result, 'mock_disabled_default_true')
        self.assertListEqual(output, ['mock_disabled_default_true'])

    def test_disable_feature_with_rollback_flag(self):
        rollback_flag = TEST_FEATURE_FLAG
        os.environ[rollback_flag] = '1'
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_feature_disabled(output)
        self.assertIsNone(result)
        self.assertListEqual(output, [])

    def test_enable_feature_with_rollback_flag_is_false(self):
        rollback_flag = TEST_FEATURE_FLAG
        os.environ[rollback_flag] = 'false'
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_feature_disabled(output)
        self.assertEqual(result, 'mock_feature_disabled')
        self.assertListEqual(output, ['mock_feature_disabled'])

    def test_ignore_enable_default_value_when_set_explicitly(self):
        feature_flag = TEST_FEATURE_FLAG
        os.environ[feature_flag] = '0'
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_enabled_default_true(output)
        self.assertIsNone(result)
        self.assertListEqual(output, [])

    def test_fail_to_enable_feature_return_default_value(self):
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_feature_default(output)
        self.assertEqual(result, FEATURE_DEFAULT)
        self.assertListEqual(output, [])

    def test_disable_feature_with_false_flag_return_default_value(self):
        feature_flag = TEST_FEATURE_FLAG
        os.environ[feature_flag] = 'false'
        mock_feature = MockFeature()
        output = []
        result = mock_feature.mock_feature_default(output)
        self.assertEqual(result, FEATURE_DEFAULT)
        self.assertListEqual(output, [])

    def test_exception_message_should_not_be_extended_on_success(self):
        mock_method = MockMethod()
        result = mock_method.mock_load_function_success()
        self.assertTrue(result)

    def test_exception_message_should_be_extended_on_subexception(self):
        mock_method = MockMethod()
        with self.assertRaises(Exception) as e:
            mock_method.mock_load_function_module_not_found()
            self.assertIn('module_not_found', e.msg)
            self.assertEqual(type(e), ModuleNotFoundError)

    def test_exception_message_should_be_extended_on_exact_exception(self):
        mock_method = MockMethod()
        with self.assertRaises(Exception) as e:
            mock_method.mock_load_function_module_not_found()
            self.assertIn('import_error', e.msg)
            self.assertEqual(type(e), ImportError)

    def test_exception_message_should_not_be_extended_on_other_exception(self):
        mock_method = MockMethod()
        with self.assertRaises(Exception) as e:
            mock_method.mock_load_function_value_error()
            self.assertNotIn('import_error', e.msg)
            self.assertEqual(type(e), ValueError)

    def test_app_settings_not_set_should_return_none(self):
        app_setting = app_setting_manager.get_app_setting(TEST_APP_SETTING_NAME)
        self.assertIsNone(app_setting)

    def test_app_settings_should_return_value(self):
        # Set application setting by os.setenv
        os.environ.update({TEST_APP_SETTING_NAME: '42'})

        # Try using utility to acquire application setting
        app_setting = app_setting_manager.get_app_setting(TEST_APP_SETTING_NAME)
        self.assertEqual(app_setting, '42')

    def test_app_settings_not_set_should_return_default_value(self):
        app_setting = app_setting_manager.get_app_setting(TEST_APP_SETTING_NAME,
                                                          'default')
        self.assertEqual(app_setting, 'default')

    def test_app_settings_should_ignore_default_value(self):
        # Set application setting by os.setenv
        os.environ.update({TEST_APP_SETTING_NAME: '42'})

        # Try using utility to acquire application setting
        app_setting = app_setting_manager.get_app_setting(TEST_APP_SETTING_NAME,
                                                          'default')
        self.assertEqual(app_setting, '42')

    def test_app_settings_should_not_trigger_validator_when_not_set(self):
        def raise_excpt(value: str):
            raise Exception('Should not raise on app setting not found')

        app_setting_manager.get_app_setting(TEST_APP_SETTING_NAME,
                                            validator=raise_excpt)

    def test_app_settings_return_default_value_when_validation_fail(self):
        def parse_int_no_raise(value: str):
            try:
                int(value)
                return True
            except ValueError:
                return False

        # Set application setting to an invalid value
        os.environ.update({TEST_APP_SETTING_NAME: 'invalid'})

        app_setting = app_setting_manager.get_app_setting(
            TEST_APP_SETTING_NAME,
            default_value='1',
            validator=parse_int_no_raise
        )

        # Because 'invalid' is not an interger, falls back to default value
        self.assertEqual(app_setting, '1')

    def test_app_settings_return_setting_value_when_validation_succeed(self):
        def parse_int_no_raise(value: str):
            try:
                int(value)
                return True
            except ValueError:
                return False

        # Set application setting to an invalid value
        os.environ.update({TEST_APP_SETTING_NAME: '42'})

        app_setting = app_setting_manager.get_app_setting(
            TEST_APP_SETTING_NAME,
            default_value='1',
            validator=parse_int_no_raise
        )

        # Because 'invalid' is not an interger, falls back to default value
        self.assertEqual(app_setting, '42')

    def test_set_get_sdk_version(self):
        test_version = '1.2.3'
        helpers.set_sdk_version(test_version)
        self.assertEqual(helpers.get_sdk_version(), test_version)

    def _unset_feature_flag(self):
        try:
            os.environ.pop(TEST_FEATURE_FLAG)
        except KeyError:
            pass



import datetime
import pytest
import unittest

from google.protobuf.timestamp_pb2 import Timestamp

import tests.protos as protos
from azure_functions_runtime_v1.bindings.nullable_converters import (
    to_nullable_bool,
    to_nullable_double,
    to_nullable_string,
    to_nullable_timestamp,
)

try:
    from http.cookies import SimpleCookie
except ImportError:
    from Cookie import SimpleCookie

headers = ['foo=bar; Path=/some/path; Secure',
           'foo2=42; Domain=123; Expires=Thu, 12-Jan-2017 13:55:08 GMT; '
           'Path=/; Max-Age=dd;']

cookies = SimpleCookie('\r\n'.join(headers))


class TestNullableConverters(unittest.TestCase):
    def test_to_nullable_string_none(self):
        self.assertEqual(to_nullable_string(None, "name", protos), None)

    def test_to_nullable_string_valid(self):
        self.assertEqual(to_nullable_string("dummy", "name", protos),
                         protos.NullableString(value="dummy"))

    def test_to_nullable_string_wrong_type(self):
        with pytest.raises(Exception) as e:
            self.assertEqual(to_nullable_string(123, "name", protos),
                             protos.NullableString(value="dummy"))
            self.assertEqual(type(e), TypeError)

    def test_to_nullable_bool_none(self):
        self.assertEqual(to_nullable_bool(None, "name", protos), None)

    def test_to_nullable_bool_valid(self):
        self.assertEqual(to_nullable_bool(True, "name", protos),
                         protos.NullableBool(value=True))

    def test_to_nullable_bool_wrong_type(self):
        with pytest.raises(Exception) as e:
            to_nullable_bool("True", "name", protos)

        self.assertEqual(e.type, TypeError)
        self.assertEqual(e.value.args[0],
                         "A 'bool' type was expected instead of a '<class "
                         "'str'>' type. "
                         "Cannot parse value True of 'name'.")

    def test_to_nullable_double_str(self):
        self.assertEqual(to_nullable_double("12", "name", protos),
                         protos.NullableDouble(value=12))

    def test_to_nullable_double_empty_str(self):
        self.assertEqual(to_nullable_double("", "name", protos), None)

    def test_to_nullable_double_invalid_str(self):
        with pytest.raises(TypeError) as e:
            to_nullable_double("222d", "name", protos)

        self.assertEqual(e.type, TypeError)
        self.assertEqual(e.value.args[0],
                         "Cannot parse value 222d of 'name' to float.")

    def test_to_nullable_double_int(self):
        self.assertEqual(to_nullable_double(12, "name", protos),
                         protos.NullableDouble(value=12))

    def test_to_nullable_double_float(self):
        self.assertEqual(to_nullable_double(12.0, "name", protos),
                         protos.NullableDouble(value=12))

    def test_to_nullable_double_none(self):
        self.assertEqual(to_nullable_double(None, "name", protos), None)

    def test_to_nullable_double_wrong_type(self):
        with pytest.raises(Exception) as e:
            to_nullable_double(object(), "name", protos)

        self.assertIn(
            "A 'int' or 'float' type was expected instead of a '<class "
            "'object'>' type",
            e.value.args[0])
        self.assertEqual(e.type, TypeError)

    def test_to_nullable_timestamp_int(self):
        self.assertEqual(to_nullable_timestamp(1000, "datetime", protos),
                         protos.NullableTimestamp(
                             value=Timestamp(seconds=int(1000))))

    def test_to_nullable_timestamp_datetime(self):
        now = datetime.datetime.now()
        self.assertEqual(to_nullable_timestamp(now, "datetime", protos),
                         protos.NullableTimestamp(
                             value=Timestamp(seconds=int(now.timestamp()))))

    def test_to_nullable_timestamp_wrong_type(self):
        with self.assertRaises(TypeError):
            to_nullable_timestamp("now", "datetime", protos)

    def test_to_nullable_timestamp_none(self):
        self.assertEqual(to_nullable_timestamp(None, "timestamp", protos), None)



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import os
import subprocess
import sys
import tempfile
import typing
import unittest

import tests.protos as protos

from azure_functions_runtime_v1.handle_event import function_environment_reload_request
from tests.utils import testutils
from tests.utils.constants import UNIT_TESTS_FOLDER
from tests.utils.mock_classes import FunctionRequest, Request, WorkerRequest

BASIC_FUNCTION_DIRECTORY = UNIT_TESTS_FOLDER / 'basic_function'


class TestGRPC(testutils.AsyncTestCase):
    pre_test_env = os.environ.copy()
    pre_test_cwd = os.getcwd()

    def _reset_environ(self):
        for key, value in self.pre_test_env.items():
            os.environ[key] = value
        os.chdir(self.pre_test_cwd)

    async def _verify_environment_reloaded(
            self,
            test_env: typing.Dict[str, str] = {},
            test_cwd: str = os.getcwd()):
        worker_request = WorkerRequest(name='function_environment_reload_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           test_cwd,
                                           test_env)),
                                       properties={'host': '123',
                                                   'protos': protos})
        result = await function_environment_reload_request(worker_request)

        status = result.result.status
        exp = result.result.exception
        self.assertEqual(status, protos.StatusResult.Success,
                         f"Exception in Reload request: {exp}")

        environ_dict = os.environ.copy()
        self.assertTrue(test_env.items() <= environ_dict.items())
        self.assertEqual(os.getcwd(), test_cwd)

        self._reset_environ()

    async def test_multiple_env_vars_load(self):
        test_env = {'TEST_KEY': 'foo', 'HELLO': 'world'}
        await self._verify_environment_reloaded(test_env=test_env)

    async def test_empty_env_vars_load(self):
        test_env = {}
        await self._verify_environment_reloaded(test_env=test_env)

    @unittest.skipIf(sys.platform == 'darwin',
                     'MacOS creates the processes specific var folder in '
                     '/private filesystem and not in /var like in linux '
                     'systems.')
    async def test_changing_current_working_directory(self):
        test_cwd = tempfile.gettempdir()
        await self._verify_environment_reloaded(test_cwd=test_cwd)

    @unittest.skipIf(sys.platform == 'darwin',
                     'MacOS creates the processes specific var folder in '
                     '/private filesystem and not in /var like in linux '
                     'systems.')
    async def test_reload_env_message(self):
        test_env = {'TEST_KEY': 'foo', 'HELLO': 'world'}
        test_cwd = tempfile.gettempdir()
        await self._verify_environment_reloaded(test_env, test_cwd)

    def _verify_sys_path_import(self, result, expected_output):
        path_import_script = os.path.join(UNIT_TESTS_FOLDER,
                                          'path_import', 'test_path_import.sh')
        try:
            subprocess.run(['chmod +x ' + path_import_script], shell=True)

            exported_path = ":".join(sys.path)
            output = subprocess.check_output(
                [path_import_script, result, exported_path],
                stderr=subprocess.STDOUT)
            decoded_output = output.decode(sys.stdout.encoding).strip()
            self.assertTrue(expected_output in decoded_output)
        finally:
            subprocess.run(['chmod -x ' + path_import_script], shell=True)
            self._reset_environ()

    # TODO
    @unittest.skip("Linux only test fails")
    def test_failed_sys_path_import(self):
        self._verify_sys_path_import(
            'fail',
            "No module named 'test_module'")

    # TODO
    @unittest.skip("Linux only test fails")
    def test_successful_sys_path_import(self):
        self._verify_sys_path_import(
            'success',
            'This module was imported!')

    def _verify_azure_namespace_import(self, result, expected_output):
        print(os.getcwd())
        path_import_script = os.path.join(UNIT_TESTS_FOLDER,
                                          'azure_namespace_import',
                                          'test_azure_namespace_import.sh')
        try:
            subprocess.run(['chmod +x ' + path_import_script], shell=True)

            output = subprocess.check_output(
                [path_import_script, result],
                stderr=subprocess.STDOUT)
            decoded_output = output.decode(sys.stdout.encoding).strip()
            self.assertTrue(expected_output in decoded_output,
                            f"Decoded Output: {decoded_output}")  # DNM
        finally:
            subprocess.run(['chmod -x ' + path_import_script], shell=True)
            self._reset_environ()



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
from unittest.mock import patch

import azure_functions_runtime_v1.handle_event as handle_event
import tests.protos as test_protos

from azure_functions_runtime_v1.handle_event import (
    worker_init_request,
    functions_metadata_request,
    function_load_request,
    function_environment_reload_request)
from tests.utils import testutils
from tests.utils.constants import UNIT_TESTS_FOLDER
from tests.utils.mock_classes import FunctionRequest, Metadata, Request, WorkerRequest

BASIC_FUNCTION_DIRECTORY = UNIT_TESTS_FOLDER / "default_template"


class TestHandleEvent(testutils.AsyncTestCase):
    @patch("azure_functions_runtime_v1.handle_event"
           ".otel_manager.get_azure_monitor_available",
           return_value=False)
    async def test_worker_init_request(self,
                                       mock_get_azure_monitor_available):
        worker_request = WorkerRequest(name='worker_init_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           BASIC_FUNCTION_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': test_protos})
        result = await worker_init_request(worker_request)
        self.assertEqual(result.capabilities, {'WorkerStatus': 'true',
                                               'RpcHttpBodyOnly': 'true',
                                               'SharedMemoryDataTransfer': 'true',
                                               'RpcHttpTriggerMetadataRemoved': 'true',
                                               'RawHttpBodyBytes': 'true',
                                               'TypedDataCollection': 'true'})
        self.assertEqual(result.worker_metadata.runtime_name, "python")
        self.assertIsNotNone(result.worker_metadata.runtime_version)
        self.assertIsNotNone(result.worker_metadata.worker_version)
        self.assertIsNotNone(result.worker_metadata.worker_bitness)
        self.assertEqual(result.result.status, 1)

    @patch("azure_functions_runtime_v1.handle_event"
           ".otel_manager.get_azure_monitor_available",
           return_value=True)
    async def test_worker_init_request_with_otel(self,
                                                 mock_otel_enabled):
        worker_request = WorkerRequest(name='worker_init_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           BASIC_FUNCTION_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': test_protos})
        result = await worker_init_request(worker_request)
        self.assertEqual('true', result.capabilities["WorkerOpenTelemetryEnabled"])
        self.assertEqual(result.worker_metadata.runtime_name, "python")
        self.assertIsNotNone(result.worker_metadata.runtime_version)
        self.assertIsNotNone(result.worker_metadata.worker_version)
        self.assertIsNotNone(result.worker_metadata.worker_bitness)
        self.assertEqual(result.result.status, 1)

    async def test_functions_metadata_request(self):
        handle_event.protos = test_protos
        metadata_result = await functions_metadata_request(None)
        self.assertEqual(metadata_result.result.status, 1)

    @patch("azure_functions_runtime_v1.handle_event._functions.get_function",
           return_value=False)
    @patch("azure_functions_runtime_v1.handle_event"
           ".load_function",
           return_value="")
    @patch("azure_functions_runtime_v1.handle_event._functions.add_function",
           return_value="")
    async def test_function_load_request(
            self,
            mock_get_function,
            mock_load_function,
            mock_add_function):
        handle_event.protos = test_protos
        worker_request = WorkerRequest(name='function_load_request',
                                       request=Request(FunctionRequest(
                                           function_id="123",
                                           metadata=Metadata(
                                               script_file=BASIC_FUNCTION_DIRECTORY,
                                               entry_point=BASIC_FUNCTION_DIRECTORY,
                                               name='hello',
                                               directory=BASIC_FUNCTION_DIRECTORY)
                                       ),
                                       ),
                                       properties={'host': '123',
                                                   'protos': test_protos})
        result = await function_load_request(worker_request)
        self.assertEqual(result.result.status, 1)

    @patch("azure_functions_runtime_v1.handle_event"
           ".otel_manager.get_azure_monitor_available",
           return_value=False)
    async def test_function_environment_reload_request(
            self,
            mock_get_azure_monitor_available):
        worker_request = WorkerRequest(name='function_environment_reload_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           BASIC_FUNCTION_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': test_protos})
        handle_event.protos = test_protos
        result = await function_environment_reload_request(worker_request)
        self.assertEqual(result.capabilities, {})
        self.assertEqual(result.worker_metadata.runtime_name, "python")
        self.assertIsNotNone(result.worker_metadata.runtime_version)
        self.assertIsNotNone(result.worker_metadata.worker_version)
        self.assertIsNotNone(result.worker_metadata.worker_bitness)
        self.assertEqual(result.result.status, 1)

    @patch("azure_functions_runtime_v1.handle_event"
           ".otel_manager.get_azure_monitor_available",
           return_value=True)
    async def test_function_environment_reload_request_with_otel(
            self,
            mock_otel_enabled):
        handle_event.protos = test_protos
        worker_request = WorkerRequest(name='function_environment_reload_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           BASIC_FUNCTION_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': test_protos})
        result = await function_environment_reload_request(worker_request)
        self.assertEqual('true', result.capabilities["WorkerOpenTelemetryEnabled"])
        self.assertEqual(result.worker_metadata.runtime_name, "python")
        self.assertIsNotNone(result.worker_metadata.runtime_version)
        self.assertIsNotNone(result.worker_metadata.worker_version)
        self.assertIsNotNone(result.worker_metadata.worker_bitness)
        self.assertEqual(result.result.status, 1)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import FunctionRpc_pb2 as FunctionRpc__pb2


class FunctionRpcStub(object):
    """Interface exported by the server.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.EventStream = channel.stream_stream(
                '/AzureFunctionsRpcMessages.FunctionRpc/EventStream',
                request_serializer=FunctionRpc__pb2.StreamingMessage.SerializeToString,
                response_deserializer=FunctionRpc__pb2.StreamingMessage.FromString,
                )


class FunctionRpcServicer(object):
    """Interface exported by the server.
    """

    def EventStream(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_FunctionRpcServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'EventStream': grpc.stream_stream_rpc_method_handler(
                    servicer.EventStream,
                    request_deserializer=FunctionRpc__pb2.StreamingMessage.FromString,
                    response_serializer=FunctionRpc__pb2.StreamingMessage.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'AzureFunctionsRpcMessages.FunctionRpc', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class FunctionRpc(object):
    """Interface exported by the server.
    """

    @staticmethod
    def EventStream(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/AzureFunctionsRpcMessages.FunctionRpc/EventStream',
            FunctionRpc__pb2.StreamingMessage.SerializeToString,
            FunctionRpc__pb2.StreamingMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: FunctionRpc.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from .identity import ClaimsIdentityRpc_pb2 as identity_dot_ClaimsIdentityRpc__pb2
from .shared import NullableTypes_pb2 as shared_dot_NullableTypes__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x46unctionRpc.proto\x12\x19\x41zureFunctionsRpcMessages\x1a\x1egoogle/protobuf/duration.proto\x1a identity/ClaimsIdentityRpc.proto\x1a\x1ashared/NullableTypes.proto\"\x8c\x11\n\x10StreamingMessage\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12>\n\x0cstart_stream\x18\x14 \x01(\x0b\x32&.AzureFunctionsRpcMessages.StartStreamH\x00\x12K\n\x13worker_init_request\x18\x11 \x01(\x0b\x32,.AzureFunctionsRpcMessages.WorkerInitRequestH\x00\x12M\n\x14worker_init_response\x18\x10 \x01(\x0b\x32-.AzureFunctionsRpcMessages.WorkerInitResponseH\x00\x12\x46\n\x10worker_heartbeat\x18\x0f \x01(\x0b\x32*.AzureFunctionsRpcMessages.WorkerHeartbeatH\x00\x12\x46\n\x10worker_terminate\x18\x0e \x01(\x0b\x32*.AzureFunctionsRpcMessages.WorkerTerminateH\x00\x12O\n\x15worker_status_request\x18\x0c \x01(\x0b\x32..AzureFunctionsRpcMessages.WorkerStatusRequestH\x00\x12Q\n\x16worker_status_response\x18\r \x01(\x0b\x32/.AzureFunctionsRpcMessages.WorkerStatusResponseH\x00\x12V\n\x19\x66ile_change_event_request\x18\x06 \x01(\x0b\x32\x31.AzureFunctionsRpcMessages.FileChangeEventRequestH\x00\x12Q\n\x16worker_action_response\x18\x07 \x01(\x0b\x32/.AzureFunctionsRpcMessages.WorkerActionResponseH\x00\x12O\n\x15\x66unction_load_request\x18\x08 \x01(\x0b\x32..AzureFunctionsRpcMessages.FunctionLoadRequestH\x00\x12Q\n\x16\x66unction_load_response\x18\t \x01(\x0b\x32/.AzureFunctionsRpcMessages.FunctionLoadResponseH\x00\x12J\n\x12invocation_request\x18\x04 \x01(\x0b\x32,.AzureFunctionsRpcMessages.InvocationRequestH\x00\x12L\n\x13invocation_response\x18\x05 \x01(\x0b\x32-.AzureFunctionsRpcMessages.InvocationResponseH\x00\x12H\n\x11invocation_cancel\x18\x15 \x01(\x0b\x32+.AzureFunctionsRpcMessages.InvocationCancelH\x00\x12\x34\n\x07rpc_log\x18\x02 \x01(\x0b\x32!.AzureFunctionsRpcMessages.RpcLogH\x00\x12j\n#function_environment_reload_request\x18\x19 \x01(\x0b\x32;.AzureFunctionsRpcMessages.FunctionEnvironmentReloadRequestH\x00\x12l\n$function_environment_reload_response\x18\x1a \x01(\x0b\x32<.AzureFunctionsRpcMessages.FunctionEnvironmentReloadResponseH\x00\x12m\n%close_shared_memory_resources_request\x18\x1b \x01(\x0b\x32<.AzureFunctionsRpcMessages.CloseSharedMemoryResourcesRequestH\x00\x12o\n&close_shared_memory_resources_response\x18\x1c \x01(\x0b\x32=.AzureFunctionsRpcMessages.CloseSharedMemoryResourcesResponseH\x00\x12Y\n\x1a\x66unctions_metadata_request\x18\x1d \x01(\x0b\x32\x33.AzureFunctionsRpcMessages.FunctionsMetadataRequestH\x00\x12Y\n\x1a\x66unction_metadata_response\x18\x1e \x01(\x0b\x32\x33.AzureFunctionsRpcMessages.FunctionMetadataResponseH\x00\x12\x64\n function_load_request_collection\x18\x1f \x01(\x0b\x32\x38.AzureFunctionsRpcMessages.FunctionLoadRequestCollectionH\x00\x12\x66\n!function_load_response_collection\x18  \x01(\x0b\x32\x39.AzureFunctionsRpcMessages.FunctionLoadResponseCollectionH\x00\x12O\n\x15worker_warmup_request\x18! \x01(\x0b\x32..AzureFunctionsRpcMessages.WorkerWarmupRequestH\x00\x12Q\n\x16worker_warmup_response\x18\" \x01(\x0b\x32/.AzureFunctionsRpcMessages.WorkerWarmupResponseH\x00\x42\t\n\x07\x63ontent\" \n\x0bStartStream\x12\x11\n\tworker_id\x18\x02 \x01(\t\"\xa6\x03\n\x11WorkerInitRequest\x12\x14\n\x0chost_version\x18\x01 \x01(\t\x12T\n\x0c\x63\x61pabilities\x18\x02 \x03(\x0b\x32>.AzureFunctionsRpcMessages.WorkerInitRequest.CapabilitiesEntry\x12W\n\x0elog_categories\x18\x03 \x03(\x0b\x32?.AzureFunctionsRpcMessages.WorkerInitRequest.LogCategoriesEntry\x12\x18\n\x10worker_directory\x18\x04 \x01(\t\x12\x1e\n\x16\x66unction_app_directory\x18\x05 \x01(\t\x1a\x33\n\x11\x43\x61pabilitiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a]\n\x12LogCategoriesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x36\n\x05value\x18\x02 \x01(\x0e\x32\'.AzureFunctionsRpcMessages.RpcLog.Level:\x02\x38\x01\"\xb5\x02\n\x12WorkerInitResponse\x12\x16\n\x0eworker_version\x18\x01 \x01(\t\x12U\n\x0c\x63\x61pabilities\x18\x02 \x03(\x0b\x32?.AzureFunctionsRpcMessages.WorkerInitResponse.CapabilitiesEntry\x12\x37\n\x06result\x18\x03 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.StatusResult\x12\x42\n\x0fworker_metadata\x18\x04 \x01(\x0b\x32).AzureFunctionsRpcMessages.WorkerMetadata\x1a\x33\n\x11\x43\x61pabilitiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x84\x02\n\x0eWorkerMetadata\x12\x14\n\x0cruntime_name\x18\x01 \x01(\t\x12\x17\n\x0fruntime_version\x18\x02 \x01(\t\x12\x16\n\x0eworker_version\x18\x03 \x01(\t\x12\x16\n\x0eworker_bitness\x18\x04 \x01(\t\x12Z\n\x11\x63ustom_properties\x18\x05 \x03(\x0b\x32?.AzureFunctionsRpcMessages.WorkerMetadata.CustomPropertiesEntry\x1a\x37\n\x15\x43ustomPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xfe\x01\n\x0cStatusResult\x12>\n\x06status\x18\x04 \x01(\x0e\x32..AzureFunctionsRpcMessages.StatusResult.Status\x12\x0e\n\x06result\x18\x01 \x01(\t\x12:\n\texception\x18\x02 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.RpcException\x12/\n\x04logs\x18\x03 \x03(\x0b\x32!.AzureFunctionsRpcMessages.RpcLog\"1\n\x06Status\x12\x0b\n\x07\x46\x61ilure\x10\x00\x12\x0b\n\x07Success\x10\x01\x12\r\n\tCancelled\x10\x02\"\x11\n\x0fWorkerHeartbeat\"B\n\x0fWorkerTerminate\x12/\n\x0cgrace_period\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xd1\x01\n\x16\x46ileChangeEventRequest\x12\x44\n\x04type\x18\x01 \x01(\x0e\x32\x36.AzureFunctionsRpcMessages.FileChangeEventRequest.Type\x12\x11\n\tfull_path\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\"P\n\x04Type\x12\x0b\n\x07Unknown\x10\x00\x12\x0b\n\x07\x43reated\x10\x01\x12\x0b\n\x07\x44\x65leted\x10\x02\x12\x0b\n\x07\x43hanged\x10\x04\x12\x0b\n\x07Renamed\x10\x08\x12\x07\n\x03\x41ll\x10\x0f\"\x91\x01\n\x14WorkerActionResponse\x12\x46\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32\x36.AzureFunctionsRpcMessages.WorkerActionResponse.Action\x12\x0e\n\x06reason\x18\x02 \x01(\t\"!\n\x06\x41\x63tion\x12\x0b\n\x07Restart\x10\x00\x12\n\n\x06Reload\x10\x01\"\x15\n\x13WorkerStatusRequest\"\x16\n\x14WorkerStatusResponse\"\xf5\x01\n FunctionEnvironmentReloadRequest\x12t\n\x15\x65nvironment_variables\x18\x01 \x03(\x0b\x32U.AzureFunctionsRpcMessages.FunctionEnvironmentReloadRequest.EnvironmentVariablesEntry\x12\x1e\n\x16\x66unction_app_directory\x18\x02 \x01(\t\x1a;\n\x19\x45nvironmentVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xbb\x02\n!FunctionEnvironmentReloadResponse\x12\x42\n\x0fworker_metadata\x18\x01 \x01(\x0b\x32).AzureFunctionsRpcMessages.WorkerMetadata\x12\x64\n\x0c\x63\x61pabilities\x18\x02 \x03(\x0b\x32N.AzureFunctionsRpcMessages.FunctionEnvironmentReloadResponse.CapabilitiesEntry\x12\x37\n\x06result\x18\x03 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.StatusResult\x1a\x33\n\x11\x43\x61pabilitiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"6\n!CloseSharedMemoryResourcesRequest\x12\x11\n\tmap_names\x18\x01 \x03(\t\"\xcb\x01\n\"CloseSharedMemoryResourcesResponse\x12m\n\x11\x63lose_map_results\x18\x01 \x03(\x0b\x32R.AzureFunctionsRpcMessages.CloseSharedMemoryResourcesResponse.CloseMapResultsEntry\x1a\x36\n\x14\x43loseMapResultsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"o\n\x1d\x46unctionLoadRequestCollection\x12N\n\x16\x66unction_load_requests\x18\x01 \x03(\x0b\x32..AzureFunctionsRpcMessages.FunctionLoadRequest\"r\n\x1e\x46unctionLoadResponseCollection\x12P\n\x17\x66unction_load_responses\x18\x01 \x03(\x0b\x32/.AzureFunctionsRpcMessages.FunctionLoadResponse\"\x90\x01\n\x13\x46unctionLoadRequest\x12\x13\n\x0b\x66unction_id\x18\x01 \x01(\t\x12@\n\x08metadata\x18\x02 \x01(\x0b\x32..AzureFunctionsRpcMessages.RpcFunctionMetadata\x12\"\n\x1amanaged_dependency_enabled\x18\x03 \x01(\x08\"\x86\x01\n\x14\x46unctionLoadResponse\x12\x13\n\x0b\x66unction_id\x18\x01 \x01(\t\x12\x37\n\x06result\x18\x02 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.StatusResult\x12 \n\x18is_dependency_downloaded\x18\x03 \x01(\x08\"\xff\x04\n\x13RpcFunctionMetadata\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x11\n\tdirectory\x18\x01 \x01(\t\x12\x13\n\x0bscript_file\x18\x02 \x01(\t\x12\x13\n\x0b\x65ntry_point\x18\x03 \x01(\t\x12N\n\x08\x62indings\x18\x06 \x03(\x0b\x32<.AzureFunctionsRpcMessages.RpcFunctionMetadata.BindingsEntry\x12\x10\n\x08is_proxy\x18\x07 \x01(\x08\x12\x37\n\x06status\x18\x08 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.StatusResult\x12\x10\n\x08language\x18\t \x01(\t\x12\x14\n\x0craw_bindings\x18\n \x03(\t\x12\x13\n\x0b\x66unction_id\x18\r \x01(\t\x12\"\n\x1amanaged_dependency_enabled\x18\x0e \x01(\x08\x12\x41\n\rretry_options\x18\x0f \x01(\x0b\x32*.AzureFunctionsRpcMessages.RpcRetryOptions\x12R\n\nproperties\x18\x10 \x03(\x0b\x32>.AzureFunctionsRpcMessages.RpcFunctionMetadata.PropertiesEntry\x1aW\n\rBindingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x35\n\x05value\x18\x02 \x01(\x0b\x32&.AzureFunctionsRpcMessages.BindingInfo:\x02\x38\x01\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\":\n\x18\x46unctionsMetadataRequest\x12\x1e\n\x16\x66unction_app_directory\x18\x01 \x01(\t\"\xcd\x01\n\x18\x46unctionMetadataResponse\x12Q\n\x19\x66unction_metadata_results\x18\x01 \x03(\x0b\x32..AzureFunctionsRpcMessages.RpcFunctionMetadata\x12\x37\n\x06result\x18\x02 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.StatusResult\x12%\n\x1duse_default_metadata_indexing\x18\x03 \x01(\x08\"\xbe\x03\n\x11InvocationRequest\x12\x15\n\rinvocation_id\x18\x01 \x01(\t\x12\x13\n\x0b\x66unction_id\x18\x02 \x01(\t\x12?\n\ninput_data\x18\x03 \x03(\x0b\x32+.AzureFunctionsRpcMessages.ParameterBinding\x12[\n\x10trigger_metadata\x18\x04 \x03(\x0b\x32\x41.AzureFunctionsRpcMessages.InvocationRequest.TriggerMetadataEntry\x12\x41\n\rtrace_context\x18\x05 \x01(\x0b\x32*.AzureFunctionsRpcMessages.RpcTraceContext\x12>\n\rretry_context\x18\x06 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.RetryContext\x1a\\\n\x14TriggerMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32$.AzureFunctionsRpcMessages.TypedData:\x02\x38\x01\"\xbf\x01\n\x0fRpcTraceContext\x12\x14\n\x0ctrace_parent\x18\x01 \x01(\t\x12\x13\n\x0btrace_state\x18\x02 \x01(\t\x12N\n\nattributes\x18\x03 \x03(\x0b\x32:.AzureFunctionsRpcMessages.RpcTraceContext.AttributesEntry\x1a\x31\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"x\n\x0cRetryContext\x12\x13\n\x0bretry_count\x18\x01 \x01(\x05\x12\x17\n\x0fmax_retry_count\x18\x02 \x01(\x05\x12:\n\texception\x18\x03 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.RpcException\"Z\n\x10InvocationCancel\x12\x15\n\rinvocation_id\x18\x02 \x01(\t\x12/\n\x0cgrace_period\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xe2\x01\n\x12InvocationResponse\x12\x15\n\rinvocation_id\x18\x01 \x01(\t\x12@\n\x0boutput_data\x18\x02 \x03(\x0b\x32+.AzureFunctionsRpcMessages.ParameterBinding\x12:\n\x0creturn_value\x18\x04 \x01(\x0b\x32$.AzureFunctionsRpcMessages.TypedData\x12\x37\n\x06result\x18\x03 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.StatusResult\"/\n\x13WorkerWarmupRequest\x12\x18\n\x10worker_directory\x18\x01 \x01(\t\"O\n\x14WorkerWarmupResponse\x12\x37\n\x06result\x18\x01 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.StatusResult\"\xfe\x04\n\tTypedData\x12\x10\n\x06string\x18\x01 \x01(\tH\x00\x12\x0e\n\x04json\x18\x02 \x01(\tH\x00\x12\x0f\n\x05\x62ytes\x18\x03 \x01(\x0cH\x00\x12\x10\n\x06stream\x18\x04 \x01(\x0cH\x00\x12\x32\n\x04http\x18\x05 \x01(\x0b\x32\".AzureFunctionsRpcMessages.RpcHttpH\x00\x12\r\n\x03int\x18\x06 \x01(\x12H\x00\x12\x10\n\x06\x64ouble\x18\x07 \x01(\x01H\x00\x12\x46\n\x10\x63ollection_bytes\x18\x08 \x01(\x0b\x32*.AzureFunctionsRpcMessages.CollectionBytesH\x00\x12H\n\x11\x63ollection_string\x18\t \x01(\x0b\x32+.AzureFunctionsRpcMessages.CollectionStringH\x00\x12H\n\x11\x63ollection_double\x18\n \x01(\x0b\x32+.AzureFunctionsRpcMessages.CollectionDoubleH\x00\x12H\n\x11\x63ollection_sint64\x18\x0b \x01(\x0b\x32+.AzureFunctionsRpcMessages.CollectionSInt64H\x00\x12I\n\x12model_binding_data\x18\x0c \x01(\x0b\x32+.AzureFunctionsRpcMessages.ModelBindingDataH\x00\x12^\n\x1d\x63ollection_model_binding_data\x18\r \x01(\x0b\x32\x35.AzureFunctionsRpcMessages.CollectionModelBindingDataH\x00\x42\x06\n\x04\x64\x61ta\"t\n\x0fRpcSharedMemory\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\x03\x12\r\n\x05\x63ount\x18\x03 \x01(\x03\x12\x34\n\x04type\x18\x04 \x01(\x0e\x32&.AzureFunctionsRpcMessages.RpcDataType\"\"\n\x10\x43ollectionString\x12\x0e\n\x06string\x18\x01 \x03(\t\" \n\x0f\x43ollectionBytes\x12\r\n\x05\x62ytes\x18\x01 \x03(\x0c\"\"\n\x10\x43ollectionDouble\x12\x0e\n\x06\x64ouble\x18\x01 \x03(\x01\"\"\n\x10\x43ollectionSInt64\x12\x0e\n\x06sint64\x18\x01 \x03(\x12\"\xab\x01\n\x10ParameterBinding\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x34\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32$.AzureFunctionsRpcMessages.TypedDataH\x00\x12G\n\x11rpc_shared_memory\x18\x03 \x01(\x0b\x32*.AzureFunctionsRpcMessages.RpcSharedMemoryH\x00\x42\n\n\x08rpc_data\"\x8b\x03\n\x0b\x42indingInfo\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x43\n\tdirection\x18\x03 \x01(\x0e\x32\x30.AzureFunctionsRpcMessages.BindingInfo.Direction\x12\x42\n\tdata_type\x18\x04 \x01(\x0e\x32/.AzureFunctionsRpcMessages.BindingInfo.DataType\x12J\n\nproperties\x18\x05 \x03(\x0b\x32\x36.AzureFunctionsRpcMessages.BindingInfo.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\'\n\tDirection\x12\x06\n\x02in\x10\x00\x12\x07\n\x03out\x10\x01\x12\t\n\x05inout\x10\x02\"=\n\x08\x44\x61taType\x12\r\n\tundefined\x10\x00\x12\n\n\x06string\x10\x01\x12\n\n\x06\x62inary\x10\x02\x12\n\n\x06stream\x10\x03\"\xe7\x04\n\x06RpcLog\x12\x15\n\rinvocation_id\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\x36\n\x05level\x18\x03 \x01(\x0e\x32\'.AzureFunctionsRpcMessages.RpcLog.Level\x12\x0f\n\x07message\x18\x04 \x01(\t\x12\x10\n\x08\x65vent_id\x18\x05 \x01(\t\x12:\n\texception\x18\x06 \x01(\x0b\x32\'.AzureFunctionsRpcMessages.RpcException\x12\x12\n\nproperties\x18\x07 \x01(\t\x12\x46\n\x0clog_category\x18\x08 \x01(\x0e\x32\x30.AzureFunctionsRpcMessages.RpcLog.RpcLogCategory\x12K\n\rpropertiesMap\x18\t \x03(\x0b\x32\x34.AzureFunctionsRpcMessages.RpcLog.PropertiesMapEntry\x1aZ\n\x12PropertiesMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32$.AzureFunctionsRpcMessages.TypedData:\x02\x38\x01\"^\n\x05Level\x12\t\n\x05Trace\x10\x00\x12\t\n\x05\x44\x65\x62ug\x10\x01\x12\x0f\n\x0bInformation\x10\x02\x12\x0b\n\x07Warning\x10\x03\x12\t\n\x05\x45rror\x10\x04\x12\x0c\n\x08\x43ritical\x10\x05\x12\x08\n\x04None\x10\x06\"8\n\x0eRpcLogCategory\x12\x08\n\x04User\x10\x00\x12\n\n\x06System\x10\x01\x12\x10\n\x0c\x43ustomMetric\x10\x02\"m\n\x0cRpcException\x12\x0e\n\x06source\x18\x03 \x01(\t\x12\x13\n\x0bstack_trace\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x19\n\x11is_user_exception\x18\x04 \x01(\x08\x12\x0c\n\x04type\x18\x05 \x01(\t\"\xf7\x02\n\rRpcHttpCookie\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\x1f\n\x06\x64omain\x18\x03 \x01(\x0b\x32\x0f.NullableString\x12\x1d\n\x04path\x18\x04 \x01(\x0b\x32\x0f.NullableString\x12#\n\x07\x65xpires\x18\x05 \x01(\x0b\x32\x12.NullableTimestamp\x12\x1d\n\x06secure\x18\x06 \x01(\x0b\x32\r.NullableBool\x12 \n\thttp_only\x18\x07 \x01(\x0b\x32\r.NullableBool\x12\x44\n\tsame_site\x18\x08 \x01(\x0e\x32\x31.AzureFunctionsRpcMessages.RpcHttpCookie.SameSite\x12 \n\x07max_age\x18\t \x01(\x0b\x32\x0f.NullableDouble\";\n\x08SameSite\x12\x08\n\x04None\x10\x00\x12\x07\n\x03Lax\x10\x01\x12\n\n\x06Strict\x10\x02\x12\x10\n\x0c\x45xplicitNone\x10\x03\"\xc5\x08\n\x07RpcHttp\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\x12@\n\x07headers\x18\x03 \x03(\x0b\x32/.AzureFunctionsRpcMessages.RpcHttp.HeadersEntry\x12\x32\n\x04\x62ody\x18\x04 \x01(\x0b\x32$.AzureFunctionsRpcMessages.TypedData\x12>\n\x06params\x18\n \x03(\x0b\x32..AzureFunctionsRpcMessages.RpcHttp.ParamsEntry\x12\x13\n\x0bstatus_code\x18\x0c \x01(\t\x12<\n\x05query\x18\x0f \x03(\x0b\x32-.AzureFunctionsRpcMessages.RpcHttp.QueryEntry\x12\"\n\x1a\x65nable_content_negotiation\x18\x10 \x01(\x08\x12\x35\n\x07rawBody\x18\x11 \x01(\x0b\x32$.AzureFunctionsRpcMessages.TypedData\x12&\n\nidentities\x18\x12 \x03(\x0b\x32\x12.RpcClaimsIdentity\x12\x39\n\x07\x63ookies\x18\x13 \x03(\x0b\x32(.AzureFunctionsRpcMessages.RpcHttpCookie\x12Q\n\x10nullable_headers\x18\x14 \x03(\x0b\x32\x37.AzureFunctionsRpcMessages.RpcHttp.NullableHeadersEntry\x12O\n\x0fnullable_params\x18\x15 \x03(\x0b\x32\x36.AzureFunctionsRpcMessages.RpcHttp.NullableParamsEntry\x12M\n\x0enullable_query\x18\x16 \x03(\x0b\x32\x35.AzureFunctionsRpcMessages.RpcHttp.NullableQueryEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a-\n\x0bParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a,\n\nQueryEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aG\n\x14NullableHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.NullableString:\x02\x38\x01\x1a\x46\n\x13NullableParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.NullableString:\x02\x38\x01\x1a\x45\n\x12NullableQueryEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.NullableString:\x02\x38\x01\"Z\n\x10ModelBindingData\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x0e\n\x06source\x18\x02 \x01(\t\x12\x14\n\x0c\x63ontent_type\x18\x03 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x04 \x01(\x0c\"e\n\x1a\x43ollectionModelBindingData\x12G\n\x12model_binding_data\x18\x01 \x03(\x0b\x32+.AzureFunctionsRpcMessages.ModelBindingData\"\xd4\x02\n\x0fRpcRetryOptions\x12\x17\n\x0fmax_retry_count\x18\x02 \x01(\x05\x12\x31\n\x0e\x64\x65lay_interval\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x33\n\x10minimum_interval\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x33\n\x10maximum_interval\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12P\n\x0eretry_strategy\x18\x06 \x01(\x0e\x32\x38.AzureFunctionsRpcMessages.RpcRetryOptions.RetryStrategy\"9\n\rRetryStrategy\x12\x17\n\x13\x65xponential_backoff\x10\x00\x12\x0f\n\x0b\x66ixed_delay\x10\x01*\xc1\x01\n\x0bRpcDataType\x12\x0b\n\x07unknown\x10\x00\x12\n\n\x06string\x10\x01\x12\x08\n\x04json\x10\x02\x12\t\n\x05\x62ytes\x10\x03\x12\n\n\x06stream\x10\x04\x12\x08\n\x04http\x10\x05\x12\x07\n\x03int\x10\x06\x12\n\n\x06\x64ouble\x10\x07\x12\x14\n\x10\x63ollection_bytes\x10\x08\x12\x15\n\x11\x63ollection_string\x10\t\x12\x15\n\x11\x63ollection_double\x10\n\x12\x15\n\x11\x63ollection_sint64\x10\x0b\x32|\n\x0b\x46unctionRpc\x12m\n\x0b\x45ventStream\x12+.AzureFunctionsRpcMessages.StreamingMessage\x1a+.AzureFunctionsRpcMessages.StreamingMessage\"\x00(\x01\x30\x01\x42\xa5\x01\n*com.microsoft.azure.functions.rpc.messagesB\rFunctionProtoP\x01Z7github.com/Azure/azure-functions-go-worker/internal/rpc\xaa\x02,Microsoft.Azure.WebJobs.Script.Grpc.Messagesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'FunctionRpc_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n*com.microsoft.azure.functions.rpc.messagesB\rFunctionProtoP\001Z7github.com/Azure/azure-functions-go-worker/internal/rpc\252\002,Microsoft.Azure.WebJobs.Script.Grpc.Messages'
  _WORKERINITREQUEST_CAPABILITIESENTRY._options = None
  _WORKERINITREQUEST_CAPABILITIESENTRY._serialized_options = b'8\001'
  _WORKERINITREQUEST_LOGCATEGORIESENTRY._options = None
  _WORKERINITREQUEST_LOGCATEGORIESENTRY._serialized_options = b'8\001'
  _WORKERINITRESPONSE_CAPABILITIESENTRY._options = None
  _WORKERINITRESPONSE_CAPABILITIESENTRY._serialized_options = b'8\001'
  _WORKERMETADATA_CUSTOMPROPERTIESENTRY._options = None
  _WORKERMETADATA_CUSTOMPROPERTIESENTRY._serialized_options = b'8\001'
  _FUNCTIONENVIRONMENTRELOADREQUEST_ENVIRONMENTVARIABLESENTRY._options = None
  _FUNCTIONENVIRONMENTRELOADREQUEST_ENVIRONMENTVARIABLESENTRY._serialized_options = b'8\001'
  _FUNCTIONENVIRONMENTRELOADRESPONSE_CAPABILITIESENTRY._options = None
  _FUNCTIONENVIRONMENTRELOADRESPONSE_CAPABILITIESENTRY._serialized_options = b'8\001'
  _CLOSESHAREDMEMORYRESOURCESRESPONSE_CLOSEMAPRESULTSENTRY._options = None
  _CLOSESHAREDMEMORYRESOURCESRESPONSE_CLOSEMAPRESULTSENTRY._serialized_options = b'8\001'
  _RPCFUNCTIONMETADATA_BINDINGSENTRY._options = None
  _RPCFUNCTIONMETADATA_BINDINGSENTRY._serialized_options = b'8\001'
  _RPCFUNCTIONMETADATA_PROPERTIESENTRY._options = None
  _RPCFUNCTIONMETADATA_PROPERTIESENTRY._serialized_options = b'8\001'
  _INVOCATIONREQUEST_TRIGGERMETADATAENTRY._options = None
  _INVOCATIONREQUEST_TRIGGERMETADATAENTRY._serialized_options = b'8\001'
  _RPCTRACECONTEXT_ATTRIBUTESENTRY._options = None
  _RPCTRACECONTEXT_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _BINDINGINFO_PROPERTIESENTRY._options = None
  _BINDINGINFO_PROPERTIESENTRY._serialized_options = b'8\001'
  _RPCLOG_PROPERTIESMAPENTRY._options = None
  _RPCLOG_PROPERTIESMAPENTRY._serialized_options = b'8\001'
  _RPCHTTP_HEADERSENTRY._options = None
  _RPCHTTP_HEADERSENTRY._serialized_options = b'8\001'
  _RPCHTTP_PARAMSENTRY._options = None
  _RPCHTTP_PARAMSENTRY._serialized_options = b'8\001'
  _RPCHTTP_QUERYENTRY._options = None
  _RPCHTTP_QUERYENTRY._serialized_options = b'8\001'
  _RPCHTTP_NULLABLEHEADERSENTRY._options = None
  _RPCHTTP_NULLABLEHEADERSENTRY._serialized_options = b'8\001'
  _RPCHTTP_NULLABLEPARAMSENTRY._options = None
  _RPCHTTP_NULLABLEPARAMSENTRY._serialized_options = b'8\001'
  _RPCHTTP_NULLABLEQUERYENTRY._options = None
  _RPCHTTP_NULLABLEQUERYENTRY._serialized_options = b'8\001'
  _globals['_RPCDATATYPE']._serialized_start=11800
  _globals['_RPCDATATYPE']._serialized_end=11993
  _globals['_STREAMINGMESSAGE']._serialized_start=143
  _globals['_STREAMINGMESSAGE']._serialized_end=2331
  _globals['_STARTSTREAM']._serialized_start=2333
  _globals['_STARTSTREAM']._serialized_end=2365
  _globals['_WORKERINITREQUEST']._serialized_start=2368
  _globals['_WORKERINITREQUEST']._serialized_end=2790
  _globals['_WORKERINITREQUEST_CAPABILITIESENTRY']._serialized_start=2644
  _globals['_WORKERINITREQUEST_CAPABILITIESENTRY']._serialized_end=2695
  _globals['_WORKERINITREQUEST_LOGCATEGORIESENTRY']._serialized_start=2697
  _globals['_WORKERINITREQUEST_LOGCATEGORIESENTRY']._serialized_end=2790
  _globals['_WORKERINITRESPONSE']._serialized_start=2793
  _globals['_WORKERINITRESPONSE']._serialized_end=3102
  _globals['_WORKERINITRESPONSE_CAPABILITIESENTRY']._serialized_start=2644
  _globals['_WORKERINITRESPONSE_CAPABILITIESENTRY']._serialized_end=2695
  _globals['_WORKERMETADATA']._serialized_start=3105
  _globals['_WORKERMETADATA']._serialized_end=3365
  _globals['_WORKERMETADATA_CUSTOMPROPERTIESENTRY']._serialized_start=3310
  _globals['_WORKERMETADATA_CUSTOMPROPERTIESENTRY']._serialized_end=3365
  _globals['_STATUSRESULT']._serialized_start=3368
  _globals['_STATUSRESULT']._serialized_end=3622
  _globals['_STATUSRESULT_STATUS']._serialized_start=3573
  _globals['_STATUSRESULT_STATUS']._serialized_end=3622
  _globals['_WORKERHEARTBEAT']._serialized_start=3624
  _globals['_WORKERHEARTBEAT']._serialized_end=3641
  _globals['_WORKERTERMINATE']._serialized_start=3643
  _globals['_WORKERTERMINATE']._serialized_end=3709
  _globals['_FILECHANGEEVENTREQUEST']._serialized_start=3712
  _globals['_FILECHANGEEVENTREQUEST']._serialized_end=3921
  _globals['_FILECHANGEEVENTREQUEST_TYPE']._serialized_start=3841
  _globals['_FILECHANGEEVENTREQUEST_TYPE']._serialized_end=3921
  _globals['_WORKERACTIONRESPONSE']._serialized_start=3924
  _globals['_WORKERACTIONRESPONSE']._serialized_end=4069
  _globals['_WORKERACTIONRESPONSE_ACTION']._serialized_start=4036
  _globals['_WORKERACTIONRESPONSE_ACTION']._serialized_end=4069
  _globals['_WORKERSTATUSREQUEST']._serialized_start=4071
  _globals['_WORKERSTATUSREQUEST']._serialized_end=4092
  _globals['_WORKERSTATUSRESPONSE']._serialized_start=4094
  _globals['_WORKERSTATUSRESPONSE']._serialized_end=4116
  _globals['_FUNCTIONENVIRONMENTRELOADREQUEST']._serialized_start=4119
  _globals['_FUNCTIONENVIRONMENTRELOADREQUEST']._serialized_end=4364
  _globals['_FUNCTIONENVIRONMENTRELOADREQUEST_ENVIRONMENTVARIABLESENTRY']._serialized_start=4305
  _globals['_FUNCTIONENVIRONMENTRELOADREQUEST_ENVIRONMENTVARIABLESENTRY']._serialized_end=4364
  _globals['_FUNCTIONENVIRONMENTRELOADRESPONSE']._serialized_start=4367
  _globals['_FUNCTIONENVIRONMENTRELOADRESPONSE']._serialized_end=4682
  _globals['_FUNCTIONENVIRONMENTRELOADRESPONSE_CAPABILITIESENTRY']._serialized_start=2644
  _globals['_FUNCTIONENVIRONMENTRELOADRESPONSE_CAPABILITIESENTRY']._serialized_end=2695
  _globals['_CLOSESHAREDMEMORYRESOURCESREQUEST']._serialized_start=4684
  _globals['_CLOSESHAREDMEMORYRESOURCESREQUEST']._serialized_end=4738
  _globals['_CLOSESHAREDMEMORYRESOURCESRESPONSE']._serialized_start=4741
  _globals['_CLOSESHAREDMEMORYRESOURCESRESPONSE']._serialized_end=4944
  _globals['_CLOSESHAREDMEMORYRESOURCESRESPONSE_CLOSEMAPRESULTSENTRY']._serialized_start=4890
  _globals['_CLOSESHAREDMEMORYRESOURCESRESPONSE_CLOSEMAPRESULTSENTRY']._serialized_end=4944
  _globals['_FUNCTIONLOADREQUESTCOLLECTION']._serialized_start=4946
  _globals['_FUNCTIONLOADREQUESTCOLLECTION']._serialized_end=5057
  _globals['_FUNCTIONLOADRESPONSECOLLECTION']._serialized_start=5059
  _globals['_FUNCTIONLOADRESPONSECOLLECTION']._serialized_end=5173
  _globals['_FUNCTIONLOADREQUEST']._serialized_start=5176
  _globals['_FUNCTIONLOADREQUEST']._serialized_end=5320
  _globals['_FUNCTIONLOADRESPONSE']._serialized_start=5323
  _globals['_FUNCTIONLOADRESPONSE']._serialized_end=5457
  _globals['_RPCFUNCTIONMETADATA']._serialized_start=5460
  _globals['_RPCFUNCTIONMETADATA']._serialized_end=6099
  _globals['_RPCFUNCTIONMETADATA_BINDINGSENTRY']._serialized_start=5961
  _globals['_RPCFUNCTIONMETADATA_BINDINGSENTRY']._serialized_end=6048
  _globals['_RPCFUNCTIONMETADATA_PROPERTIESENTRY']._serialized_start=6050
  _globals['_RPCFUNCTIONMETADATA_PROPERTIESENTRY']._serialized_end=6099
  _globals['_FUNCTIONSMETADATAREQUEST']._serialized_start=6101
  _globals['_FUNCTIONSMETADATAREQUEST']._serialized_end=6159
  _globals['_FUNCTIONMETADATARESPONSE']._serialized_start=6162
  _globals['_FUNCTIONMETADATARESPONSE']._serialized_end=6367
  _globals['_INVOCATIONREQUEST']._serialized_start=6370
  _globals['_INVOCATIONREQUEST']._serialized_end=6816
  _globals['_INVOCATIONREQUEST_TRIGGERMETADATAENTRY']._serialized_start=6724
  _globals['_INVOCATIONREQUEST_TRIGGERMETADATAENTRY']._serialized_end=6816
  _globals['_RPCTRACECONTEXT']._serialized_start=6819
  _globals['_RPCTRACECONTEXT']._serialized_end=7010
  _globals['_RPCTRACECONTEXT_ATTRIBUTESENTRY']._serialized_start=6961
  _globals['_RPCTRACECONTEXT_ATTRIBUTESENTRY']._serialized_end=7010
  _globals['_RETRYCONTEXT']._serialized_start=7012
  _globals['_RETRYCONTEXT']._serialized_end=7132
  _globals['_INVOCATIONCANCEL']._serialized_start=7134
  _globals['_INVOCATIONCANCEL']._serialized_end=7224
  _globals['_INVOCATIONRESPONSE']._serialized_start=7227
  _globals['_INVOCATIONRESPONSE']._serialized_end=7453
  _globals['_WORKERWARMUPREQUEST']._serialized_start=7455
  _globals['_WORKERWARMUPREQUEST']._serialized_end=7502
  _globals['_WORKERWARMUPRESPONSE']._serialized_start=7504
  _globals['_WORKERWARMUPRESPONSE']._serialized_end=7583
  _globals['_TYPEDDATA']._serialized_start=7586
  _globals['_TYPEDDATA']._serialized_end=8224
  _globals['_RPCSHAREDMEMORY']._serialized_start=8226
  _globals['_RPCSHAREDMEMORY']._serialized_end=8342
  _globals['_COLLECTIONSTRING']._serialized_start=8344
  _globals['_COLLECTIONSTRING']._serialized_end=8378
  _globals['_COLLECTIONBYTES']._serialized_start=8380
  _globals['_COLLECTIONBYTES']._serialized_end=8412
  _globals['_COLLECTIONDOUBLE']._serialized_start=8414
  _globals['_COLLECTIONDOUBLE']._serialized_end=8448
  _globals['_COLLECTIONSINT64']._serialized_start=8450
  _globals['_COLLECTIONSINT64']._serialized_end=8484
  _globals['_PARAMETERBINDING']._serialized_start=8487
  _globals['_PARAMETERBINDING']._serialized_end=8658
  _globals['_BINDINGINFO']._serialized_start=8661
  _globals['_BINDINGINFO']._serialized_end=9056
  _globals['_BINDINGINFO_PROPERTIESENTRY']._serialized_start=6050
  _globals['_BINDINGINFO_PROPERTIESENTRY']._serialized_end=6099
  _globals['_BINDINGINFO_DIRECTION']._serialized_start=8954
  _globals['_BINDINGINFO_DIRECTION']._serialized_end=8993
  _globals['_BINDINGINFO_DATATYPE']._serialized_start=8995
  _globals['_BINDINGINFO_DATATYPE']._serialized_end=9056
  _globals['_RPCLOG']._serialized_start=9059
  _globals['_RPCLOG']._serialized_end=9674
  _globals['_RPCLOG_PROPERTIESMAPENTRY']._serialized_start=9430
  _globals['_RPCLOG_PROPERTIESMAPENTRY']._serialized_end=9520
  _globals['_RPCLOG_LEVEL']._serialized_start=9522
  _globals['_RPCLOG_LEVEL']._serialized_end=9616
  _globals['_RPCLOG_RPCLOGCATEGORY']._serialized_start=9618
  _globals['_RPCLOG_RPCLOGCATEGORY']._serialized_end=9674
  _globals['_RPCEXCEPTION']._serialized_start=9676
  _globals['_RPCEXCEPTION']._serialized_end=9785
  _globals['_RPCHTTPCOOKIE']._serialized_start=9788
  _globals['_RPCHTTPCOOKIE']._serialized_end=10163
  _globals['_RPCHTTPCOOKIE_SAMESITE']._serialized_start=10104
  _globals['_RPCHTTPCOOKIE_SAMESITE']._serialized_end=10163
  _globals['_RPCHTTP']._serialized_start=10166
  _globals['_RPCHTTP']._serialized_end=11259
  _globals['_RPCHTTP_HEADERSENTRY']._serialized_start=10904
  _globals['_RPCHTTP_HEADERSENTRY']._serialized_end=10950
  _globals['_RPCHTTP_PARAMSENTRY']._serialized_start=10952
  _globals['_RPCHTTP_PARAMSENTRY']._serialized_end=10997
  _globals['_RPCHTTP_QUERYENTRY']._serialized_start=10999
  _globals['_RPCHTTP_QUERYENTRY']._serialized_end=11043
  _globals['_RPCHTTP_NULLABLEHEADERSENTRY']._serialized_start=11045
  _globals['_RPCHTTP_NULLABLEHEADERSENTRY']._serialized_end=11116
  _globals['_RPCHTTP_NULLABLEPARAMSENTRY']._serialized_start=11118
  _globals['_RPCHTTP_NULLABLEPARAMSENTRY']._serialized_end=11188
  _globals['_RPCHTTP_NULLABLEQUERYENTRY']._serialized_start=11190
  _globals['_RPCHTTP_NULLABLEQUERYENTRY']._serialized_end=11259
  _globals['_MODELBINDINGDATA']._serialized_start=11261
  _globals['_MODELBINDINGDATA']._serialized_end=11351
  _globals['_COLLECTIONMODELBINDINGDATA']._serialized_start=11353
  _globals['_COLLECTIONMODELBINDINGDATA']._serialized_end=11454
  _globals['_RPCRETRYOPTIONS']._serialized_start=11457
  _globals['_RPCRETRYOPTIONS']._serialized_end=11797
  _globals['_RPCRETRYOPTIONS_RETRYSTRATEGY']._serialized_start=11740
  _globals['_RPCRETRYOPTIONS_RETRYSTRATEGY']._serialized_end=11797
  _globals['_FUNCTIONRPC']._serialized_start=11995
  _globals['_FUNCTIONRPC']._serialized_end=12119
# @@protoc_insertion_point(module_scope)



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

from .handle_event import (worker_init_request,
                           functions_metadata_request,
                           function_environment_reload_request,
                           invocation_request,
                           function_load_request)
from .utils.threadpool import (
    start_threadpool_executor,
    stop_threadpool_executor,
    get_threadpool_executor,
)

__all__ = ('worker_init_request',
           'functions_metadata_request',
           'function_environment_reload_request',
           'invocation_request',
           'function_load_request',
           'start_threadpool_executor',
           'stop_threadpool_executor',
           'get_threadpool_executor')



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import sys

# Constants for Azure Functions Python Worker
CUSTOMER_PACKAGES_PATH = "/home/site/wwwroot/.python_packages/lib/site" \
                         "-packages"
HTTP = "http"
HTTP_TRIGGER = "httpTrigger"
METADATA_PROPERTIES_WORKER_INDEXED = "worker_indexed"
MODULE_NOT_FOUND_TS_URL = "https://aka.ms/functions-modulenotfound"
PYTHON_LANGUAGE_RUNTIME = "python"
RETRY_POLICY = "retry_policy"
TRUE = "true"
TRACEPARENT = "traceparent"
TRACESTATE = "tracestate"
X_MS_INVOCATION_ID = "x-ms-invocation-id"


# Capabilities
FUNCTION_DATA_CACHE = "FunctionDataCache"
HTTP_URI = "HttpUri"
RAW_HTTP_BODY_BYTES = "RawHttpBodyBytes"
REQUIRES_ROUTE_PARAMETERS = "RequiresRouteParameters"
RPC_HTTP_BODY_ONLY = "RpcHttpBodyOnly"
RPC_HTTP_TRIGGER_METADATA_REMOVED = "RpcHttpTriggerMetadataRemoved"
SHARED_MEMORY_DATA_TRANSFER = "SharedMemoryDataTransfer"
TYPED_DATA_COLLECTION = "TypedDataCollection"
# When this capability is enabled, logs are not piped back to the
# host from the worker. Logs will directly go to where the user has
# configured them to go. This is to ensure that the logs are not
# duplicated.
WORKER_OPEN_TELEMETRY_ENABLED = "WorkerOpenTelemetryEnabled"
WORKER_STATUS = "WorkerStatus"


# Platform Environment Variables
AZURE_WEBJOBS_SCRIPT_ROOT = "AzureWebJobsScriptRoot"
CONTAINER_NAME = "CONTAINER_NAME"


# Python Specific Feature Flags and App Settings
# Appsetting to specify AppInsights connection string
APPLICATIONINSIGHTS_CONNECTION_STRING = "APPLICATIONINSIGHTS_CONNECTION_STRING"
# Appsetting to turn on ApplicationInsights support/features
# A value of "true" enables the setting
PYTHON_APPLICATIONINSIGHTS_ENABLE_TELEMETRY = \
    "PYTHON_APPLICATIONINSIGHTS_ENABLE_TELEMETRY"
# Appsetting to specify root logger name of logger to collect telemetry for
# Used by Azure monitor distro (Application Insights)
PYTHON_APPLICATIONINSIGHTS_LOGGER_NAME = "PYTHON_APPLICATIONINSIGHTS_LOGGER_NAME"
PYTHON_APPLICATIONINSIGHTS_LOGGER_NAME_DEFAULT = ""
PYTHON_ENABLE_DEBUG_LOGGING = "PYTHON_ENABLE_DEBUG_LOGGING"
# Appsetting to turn on OpenTelemetry support/features
# A value of "true" enables the setting
PYTHON_ENABLE_OPENTELEMETRY = "PYTHON_ENABLE_OPENTELEMETRY"
PYTHON_THREADPOOL_THREAD_COUNT = "PYTHON_THREADPOOL_THREAD_COUNT"
PYTHON_THREADPOOL_THREAD_COUNT_DEFAULT = 1
PYTHON_THREADPOOL_THREAD_COUNT_MAX = sys.maxsize
PYTHON_THREADPOOL_THREAD_COUNT_MIN = 1



# Copyright (c) Microsoft Corporation. All rights reserved.
import os
import unittest

import tests.protos as protos

from azure_functions_runtime_v1.handle_event import otel_manager, worker_init_request
from azure_functions_runtime_v1.otel import (initialize_azure_monitor,
                                             update_opentelemetry_status)
from azure_functions_runtime_v1.logging import logger
from tests.utils.constants import UNIT_TESTS_FOLDER
from tests.utils.mock_classes import FunctionRequest, Request, WorkerRequest
from unittest.mock import MagicMock, patch


FUNCTION_APP_DIRECTORY = UNIT_TESTS_FOLDER / 'basic_functions'


class TestOpenTelemetry(unittest.TestCase):

    def test_update_opentelemetry_status_import_error(self):
        with patch.dict('sys.modules', {
            'opentelemetry': None,
            'opentelemetry.context': None,
            'opentelemetry.trace': None,
            'opentelemetry.trace.propagation': None,
            'opentelemetry.trace.propagation.tracecontext': None,
        }):
            # Verify that context variables are None due to ImportError
            with self.assertLogs(logger.name, 'ERROR') as cm:
                update_opentelemetry_status()
                self.assertTrue(
                    any("Cannot import OpenTelemetry libraries."
                        in message for message in cm.output)
                )

    @patch('builtins.__import__')
    def test_update_opentelemetry_status_success(
            self, mock_imports):
        mock_imports.return_value = MagicMock()
        update_opentelemetry_status()
        self.assertIsNotNone(otel_manager.get_context_api())
        self.assertIsNotNone(otel_manager.get_trace_context_propagator())

    @patch('builtins.__import__')
    @patch("azure_functions_runtime_v1.otel.update_opentelemetry_status")
    def test_initialize_azure_monitor_success(
        self,
        mock_update_ot,
        mock_imports,
    ):
        mock_imports.return_value = MagicMock()
        initialize_azure_monitor()
        mock_update_ot.assert_called_once()
        self.assertTrue(otel_manager.get_azure_monitor_available())

    @patch("azure_functions_runtime_v1.otel.update_opentelemetry_status")
    def test_initialize_azure_monitor_import_error(
        self,
        mock_update_ot,
    ):
        with patch('builtins.__import__', side_effect=ImportError):
            initialize_azure_monitor()
            mock_update_ot.assert_called_once()
            # Verify that azure_monitor_available is set to False due to ImportError
            self.assertFalse(otel_manager.get_azure_monitor_available())

    @patch.dict(os.environ, {'PYTHON_APPLICATIONINSIGHTS_ENABLE_TELEMETRY': 'true'})
    @patch('builtins.__import__')
    async def test_init_request_initialize_azure_monitor_enabled_app_setting(
            self,
            mock_imports,
    ):
        mock_imports.return_value = MagicMock()

        worker_request = WorkerRequest(name='worker_init_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           FUNCTION_APP_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': protos})
        init_response = await worker_init_request(worker_request)

        self.assertEqual(init_response.result.status,
                         protos.StatusResult.Success)

        # Verify azure_monitor_available is set to True
        self.assertTrue(otel_manager.get_azure_monitor_available())
        # Verify that WorkerOpenTelemetryEnabled capability is set to _TRUE
        capabilities = init_response.capabilities
        self.assertIn("WorkerOpenTelemetryEnabled", capabilities)
        self.assertEqual(capabilities["WorkerOpenTelemetryEnabled"], "true")

    @patch("azure_functions_runtime.handle_event."
           "otel_manager.initialize_azure_monitor")
    async def test_init_request_initialize_azure_monitor_default_app_setting(
        self,
        mock_initialize_azmon,
    ):
        worker_request = WorkerRequest(name='worker_init_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           FUNCTION_APP_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': protos})
        init_response = await worker_init_request(worker_request)

        self.assertEqual(init_response.result.status,
                         protos.StatusResult.Success)

        # Azure monitor initialized not called
        # Since default behavior is not enabled
        mock_initialize_azmon.assert_not_called()

        # Verify azure_monitor_available is set to False
        self.assertFalse(otel_manager.get_azure_monitor_available())
        # Verify that WorkerOpenTelemetryEnabled capability is not set
        capabilities = init_response.capabilities
        self.assertNotIn("WorkerOpenTelemetryEnabled", capabilities)

    @patch.dict(os.environ, {'PYTHON_APPLICATIONINSIGHTS_ENABLE_TELEMETRY': 'false'})
    @patch("azure_functions_runtime.otel_manager.initialize_azure_monitor")
    async def test_init_request_initialize_azure_monitor_disabled_app_setting(
        self,
        mock_initialize_azmon,
    ):
        worker_request = WorkerRequest(name='worker_init_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           FUNCTION_APP_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': protos})
        init_response = await worker_init_request(worker_request)

        self.assertEqual(init_response.result.status,
                         protos.StatusResult.Success)

        # Azure monitor initialized not called
        mock_initialize_azmon.assert_not_called()

        # Verify azure_monitor_available is set to False
        self.assertFalse(otel_manager.get_azure_monitor_available())
        # Verify that WorkerOpenTelemetryEnabled capability is not set
        capabilities = init_response.capabilities
        self.assertNotIn("WorkerOpenTelemetryEnabled", capabilities)

    @patch.dict(os.environ, {'PYTHON_ENABLE_OPENTELEMETRY': 'true'})
    async def test_init_request_enable_opentelemetry_enabled_app_setting(
        self,
    ):
        worker_request = WorkerRequest(name='worker_init_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           FUNCTION_APP_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': protos})
        init_response = await worker_init_request(worker_request)

        self.assertEqual(init_response.result.status,
                         protos.StatusResult.Success)

        # Verify otel_libs_available is set to True
        self.assertTrue(otel_manager.get_azure_monitor_available())
        # Verify that WorkerOpenTelemetryEnabled capability is set to _TRUE
        capabilities = init_response.capabilities
        self.assertIn("WorkerOpenTelemetryEnabled", capabilities)
        self.assertEqual(capabilities["WorkerOpenTelemetryEnabled"], "true")

    @patch.dict(os.environ, {'PYTHON_ENABLE_OPENTELEMETRY': 'false'})
    async def test_init_request_enable_opentelemetry_default_app_setting(
        self,
    ):
        worker_request = WorkerRequest(name='worker_init_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           FUNCTION_APP_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': protos})
        init_response = await worker_init_request(worker_request)

        self.assertEqual(init_response.result.status,
                         protos.StatusResult.Success)

        # Verify otel_libs_available is set to False by default
        self.assertFalse(otel_manager.get_otel_libs_available())
        # Verify that WorkerOpenTelemetryEnabled capability is not set
        capabilities = init_response.capabilities
        self.assertNotIn("WorkerOpenTelemetryEnabled", capabilities)

    @patch.dict(os.environ, {'PYTHON_APPLICATIONINSIGHTS_ENABLE_TELEMETRY': 'false'})
    async def test_init_request_enable_azure_monitor_disabled_app_setting(
        self,
    ):
        worker_request = WorkerRequest(name='worker_init_request',
                                       request=Request(FunctionRequest(
                                           'hello',
                                           FUNCTION_APP_DIRECTORY)),
                                       properties={'host': '123',
                                                   'protos': protos})
        init_response = await worker_init_request(worker_request)

        self.assertEqual(init_response.result.status,
                         protos.StatusResult.Success)

        # Verify otel_libs_available is set to False by default
        self.assertFalse(otel_manager.get_azure_monitor_available())
        # Verify that WorkerOpenTelemetryEnabled capability is not set
        capabilities = init_response.capabilities
        self.assertNotIn("WorkerOpenTelemetryEnabled", capabilities)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identity/ClaimsIdentityRpc.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..shared import NullableTypes_pb2 as shared_dot_NullableTypes__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n identity/ClaimsIdentityRpc.proto\x1a\x1ashared/NullableTypes.proto\"\xb0\x01\n\x11RpcClaimsIdentity\x12,\n\x13\x61uthentication_type\x18\x01 \x01(\x0b\x32\x0f.NullableString\x12(\n\x0fname_claim_type\x18\x02 \x01(\x0b\x32\x0f.NullableString\x12(\n\x0frole_claim_type\x18\x03 \x01(\x0b\x32\x0f.NullableString\x12\x19\n\x06\x63laims\x18\x04 \x03(\x0b\x32\t.RpcClaim\"\'\n\x08RpcClaim\x12\r\n\x05value\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\tB,\n*com.microsoft.azure.functions.rpc.messagesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'identity.ClaimsIdentityRpc_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n*com.microsoft.azure.functions.rpc.messages'
  _globals['_RPCCLAIMSIDENTITY']._serialized_start=65
  _globals['_RPCCLAIMSIDENTITY']._serialized_end=241
  _globals['_RPCCLAIM']._serialized_start=243
  _globals['_RPCCLAIM']._serialized_end=282
# @@protoc_insertion_point(module_scope)



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import pathlib

PROJECT_ROOT = pathlib.Path(__file__).parent.parent.parent
TESTS_ROOT = PROJECT_ROOT / 'tests'
UNIT_TESTS_FOLDER = TESTS_ROOT / pathlib.Path('unittests')



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: shared/NullableTypes.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1ashared/NullableTypes.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"+\n\x0eNullableString\x12\x0f\n\x05value\x18\x01 \x01(\tH\x00\x42\x08\n\x06string\"+\n\x0eNullableDouble\x12\x0f\n\x05value\x18\x01 \x01(\x01H\x00\x42\x08\n\x06\x64ouble\"\'\n\x0cNullableBool\x12\x0f\n\x05value\x18\x01 \x01(\x08H\x00\x42\x06\n\x04\x62ool\"M\n\x11NullableTimestamp\x12+\n\x05value\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x42\x0b\n\ttimestampB,\n*com.microsoft.azure.functions.rpc.messagesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'shared.NullableTypes_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n*com.microsoft.azure.functions.rpc.messages'
  _globals['_NULLABLESTRING']._serialized_start=63
  _globals['_NULLABLESTRING']._serialized_end=106
  _globals['_NULLABLEDOUBLE']._serialized_start=108
  _globals['_NULLABLEDOUBLE']._serialized_end=151
  _globals['_NULLABLEBOOL']._serialized_start=153
  _globals['_NULLABLEBOOL']._serialized_end=192
  _globals['_NULLABLETIMESTAMP']._serialized_start=194
  _globals['_NULLABLETIMESTAMP']._serialized_end=271
# @@protoc_insertion_point(module_scope)



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import os

from azure_functions_runtime_v1.utils.app_setting_manager import (
    get_python_appsetting_state)
from azure_functions_runtime_v1.utils.constants import (
    PYTHON_ENABLE_DEBUG_LOGGING,
    PYTHON_THREADPOOL_THREAD_COUNT,
)
from tests.utils import testutils
from unittest.mock import patch


class TestDefaultAppSettingsLogs(testutils.AsyncTestCase):
    """Tests for default app settings logs."""

    def test_get_python_appsetting_state(self):
        app_setting_state = get_python_appsetting_state()
        expected_string = ""
        self.assertEqual(expected_string, app_setting_state)


class TestNonDefaultAppSettingsLogs(testutils.AsyncTestCase):
    """Tests for non-default app settings logs."""

    @classmethod
    def setUpClass(cls):
        os_environ = os.environ.copy()
        os_environ[PYTHON_THREADPOOL_THREAD_COUNT] = '20'
        os_environ[PYTHON_ENABLE_DEBUG_LOGGING] = '1'
        cls._patch_environ = patch.dict('os.environ', os_environ)
        cls._patch_environ.start()
        super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        cls._patch_environ.stop()

    def test_get_python_appsetting_state(self):
        app_setting_state = get_python_appsetting_state()
        self.assertIn("PYTHON_THREADPOOL_THREAD_COUNT: 20 | ",
                      app_setting_state)
        self.assertIn("PYTHON_ENABLE_DEBUG_LOGGING: 1 | ", app_setting_state)
